/*
********************************************************************
**                                                                **
**	Tab Counter  -  Extension for Pale Moon                       **
**                                                                **
**	Last edit: September 13, 2018                                 **
**                                                                **
**	Release number: 1                                             **
**                                                                **
********************************************************************
*/
(function(global) {
	let positions = {};

	/*
	* Assigns position that will be used by `restorePosition`
	* if the button is not found on any toolbar's current set.
	* If `beforeID` is null, or no such item is found on the toolbar,
	* the button will be added to the end.
	* @param beforeID ID of the element before which the button will be inserted.
	*/
	global.setDefaultPosition = function(buttonID, toolbarID, beforeID) {
		positions[buttonID] = [toolbarID, beforeID];
	};
  
	/*
	* Restores the button's saved position.
	* @param {XULDocument} doc - XUL window document.
	* @param {XULElement} button - button element or array of button elements.
	*/
	global.restorePosition = function(doc, button) {
		function $(sel, all)
			doc[all ? "querySelectorAll" : "getElementById"](sel);

		// If array of button, add them all at once in correct order.
		if(Array.isArray(button)) {
			let pendingAdds = [];
			for(let i=0; i < button.length; i++) {
				let params = findRestoreLocation(doc, button[i]);
				// If new install, just add the button, otherwise save to add later
				if(params.newInstall) {
					performRestore(doc, button[i], params);
				} else {
					pendingAdds.push({buttonIdx: i, params: params});
				};
			};
			if(pendingAdds.length > 0) {
				// Sort so rightmost buttons get added first.
				pendingAdds = pendingAdds.sort(function(a,b) {
					return a.params.idx > b.params.idx;
				});
				for(let i=0; i < pendingAdds.length; i++) {
					performRestore(doc, button[pendingAdds[i].buttonIdx], pendingAdds[i].params);
				};
			};
		} else {
			let params = findRestoreLocation(doc, button);
			performRestore(doc, button, params);
		};
	};

	function findRestoreLocation(doc, button) {
		function $(sel, all)
			doc[all ? "querySelectorAll" : "getElementById"](sel);

		let toolbar, currentset, idx, newInstall = false,
			toolbars = $("toolbar", true);
		for(let i = 0; i < toolbars.length; ++i) {
			let tb = toolbars[i];
			currentset = tb.getAttribute("currentset").split(",");
			idx = currentset.indexOf(button.id);
			if(idx != -1) {
				toolbar = tb;
				break;
			};
		};

		// Saved position not found, using the default one, if any.
		if(!toolbar && (button.id in positions)) {
			let [tbID, beforeID] = positions[button.id];
			toolbar = $(tbID);
			[currentset, idx] = persist(doc, toolbar, button.id, beforeID);
			newInstall = true;
		};
		return { toolbarId: (toolbar ? toolbar.id : null), currentset: currentset, idx: idx, newInstall: newInstall };
	};

	function performRestore(doc, button, aParams) {
		function $(sel, all)
			doc[all ? "querySelectorAll" : "getElementById"](sel);

		// Add to Toolbar palette
		$("navigator-toolbox").palette.appendChild(button);

		let toolbar = (aParams.toolbarId ? $(aParams.toolbarId) : null), currentset = aParams.currentset, 
			idx = aParams.idx, newInstall = aParams.newInstall;
    
		if(toolbar) {
			// If using add-on toolbar and it's hidden, show it
			if((toolbar.id == "addon-bar") && (toolbar.getAttribute("collapsed") == "true")) {
				toolbar.setAttribute("collapsed", "false");
			};
			let before = null;
			if(idx != -1) {
				// Need to get ids for separators so can insert in front of them if need be.
				let separators = toolbar.getElementsByTagName("toolbarseparator");
				let j = 0;
				for(let i =0; ((i < currentset.length) && (j < separators.length)); ++i) {
					if(currentset[i] == "separator") {
						currentset[i] = separators[j++].id
					};
				};
				// Inserting the button before the first item in `currentset`
				// after `idx` that is present in the document.
				for(let i = idx + 1; i < currentset.length; ++i) {
					before = $(currentset[i]);
					if(before) {
						break;
					};
				};
			};
			toolbar.insertItem(button.id, before);
			if(newInstall) {
				toolbar.setAttribute("currentset", toolbar.currentSet);
				doc.persist(toolbar.id, "currentset");
			};

			// Remove from positions so doesn't keep getting added in new windows.
			delete positions[button.id];
		};
	};

	function persist(document, toolbar, buttonID, beforeID) {
		let currentset = toolbar.currentSet.split(","),
			idx = (beforeID && currentset.indexOf(beforeID)) || -1;
		if(idx != -1) {
			currentset.splice(idx, 0, buttonID);
		} else {
			currentset.push(buttonID);
		};
		return [currentset, idx];
	};
})(this);
