/*
********************************************************************
**                                                                **
**	Tab Counter  -  Extension for Pale Moon                       **
**                                                                **
**	Last edit: September 10, 2018                                 **
**                                                                **
**	Release number: 1                                             **
**                                                                **
********************************************************************
*/

/**
 * Save callbacks to run when unloading. Optionally scope the callback to a
 * container, e.g., window. Provide a way to run all the callbacks.
 *
 * @usage unload(): Run all callbacks and release them.
 *
 * @usage unload(callback): Add a callback to run on unload.
 * @param [function] callback: 0-parameter function to call on unload.
 * @return [function]: A 0-parameter function that undoes adding the callback.
 *
 * @usage unload(callback, container) Add a scoped callback to run on unload.
 * @param [function] callback: 0-parameter function to call on unload.
 * @param [node] container: Remove the callback when this container unloads.
 * @return [function]: A 0-parameter function that undoes adding the callback.
 */
exports.unload = function unload(callback, container) {
	// Initialize the array of unloaders on the first usage.
	let unloaders = unload.unloaders;
	if(unloaders == null) {
		unloaders = unload.unloaders = [];
	};
	// Calling with no arguments runs all the unloader callbacks.
	if(callback == null) {
		unloaders.slice().forEach(function(unloader) unloader());
		unloaders.length = 0;
		return;
	};

	// The callback is bound to the lifetime of the container if we have one.
	if(container != null) {
		// Remove the unloader when the container unloads.
		container.addEventListener("unload", unloader, false);

		// Wrap the callback to additionally remove the unload listener.
		let origCallback = callback;
		callback = function(aParam) {
			container.removeEventListener("unload", unloader, false);
			removeUnloader();
			origCallback(aParam);
		};
	};

	// Wrap the callback in a function that ignores failures.
	function unloader(aEvent) {
		try{
			var winClose = false;
			if(aEvent) {
				if((aEvent.type == "unload") && this.ChromeWindow) {
					winClose = true;
				};
			};
			callback(winClose);
		} catch(ex) {
			Components.utils.reportError("Tab Counter exception: " + ex + "\n" + ex.stack);
		};
	};
	unloaders.push(unloader);

	// Provide a way to remove the unloader.
	function removeUnloader() {
		let index = unloaders.indexOf(unloader);
		if(index != -1) {
			unloaders.splice(index, 1);
		};
	};
	return removeUnloader;
};
