/*
********************************************************************
**                                                                **
**	Tab Counter  -  Extension for Pale Moon                       **
**                                                                **
**	Last edit: September 10, 2018                                 **
**                                                                **
**	Release number: 1                                             **
**                                                                **
********************************************************************
*/
var {unload} = require("unload");

/**
 * Waits for a browser window to finish loading before running the callback
 *
 * @usage runOnLoad(window, callback): Apply a callback to run on a window when it loads.
 * @param [function] callback: 1-parameter function that gets a browser window.
 * @param [function] winType: a parameter that defines what kind of window is "browser window".
 */
exports.runOnLoad = function runOnLoad(window, callback, winType) {
	// Listen for one load event before checking the window type.
	window.addEventListener("load", function() {
		window.removeEventListener("load", arguments.callee, false);

		// Now that the window has loaded, only handle browser windows.
		if(window.document.documentElement.getAttribute("windowtype") == winType) {
			Components.utils.import("chrome://tabcounter-pm/content/tbcGlobal.jsm", this);
			let sum = this.globalVal.getSetVal("openWindows", "getV") + 1;
			this.globalVal.getSetVal("openWindows", "setV", sum); // set value.
			let loadedWin = this.globalVal.getSetVal("openWindows", "getV"); // get value.
			if(loadedWin == this.globalVal.getSetVal("restoredWindows", "getV") &&
				!this.globalVal.getSetVal("browserIsBusy", "getV")) { // get value.
				// On browser startup with more than 1 windows, only the last will run its
				// iterator to update the global counts in other windows.
				// This method does not work on Firefox, due to different timings in
				// events/obs notifications we use.
				this.globalVal.getSetVal("canUpdateOtherWin", "setT"); // set true.
				this.globalVal.getSetVal("restoredWindows", "setV", 0); // set value.
			};
			callback(window);
		} else if(window.document.getElementById("CustomizeToolbarWindow")) {
			window.opener.gTabCounterPm.onCustomize();
		};
	}, false);
};

/**
 * Add functionality to existing browser windows
 *
 * @usage runOnWindows(callback): Apply a callback to each open browser window.
 * @param [function] callback: 1-parameter function that gets a browser window.
 * @param [function] winType: a parameter that defines what kind of window is "browser window".
 */
exports.runOnWindows = function runOnWindows(callback, winType) {
	// Wrap the callback in a function that ignores failures.
	function watcher(window) {
		try{
			callback(window);
		} catch(ex) {
			window.console.log("Tab Counter exception: " + ex);
		};
	};

	var addExistingWin = 0;

	// Add functionality to existing windows.
	let browserWindows = Services.wm.getEnumerator(winType);
	while(browserWindows.hasMoreElements()) {
		addExistingWin++;
		// Only run the watcher immediately if the browser is completely loaded.
		let browserWindow = browserWindows.getNext();
		if(browserWindow.document.readyState == "complete") {
			watcher(browserWindow);
		} else { // Wait for the window to load before continuing.
			runOnLoad(browserWindow, watcher, winType);
		};
	};
	Components.utils.import("chrome://tabcounter-pm/content/tbcGlobal.jsm", this);
	this.globalVal.getSetVal("openWindows", "setV", addExistingWin); // set value.
	addExistingWin = 0;
};

/**
 * Apply a callback to each open and new browser windows.
 *
 * @usage watchWindows(callback): Apply a callback to each browser window.
 * @param [function] callback: 1-parameter function that gets a browser window.
 * @param [function] winType: a parameter that defines what kind of window is "browser window". 
 */
exports.watchWindows = function watchWindows(callback, winType) {

	// Wrap the callback in a function that ignores failures.
	function watcher(window) {
		try{
			callback(window);
		} catch(ex) {
			window.console.log("Tab Counter exception: " + ex);
		};
	};

	// Add functionality to existing windows.
	runOnWindows(callback, winType);

	// Watch for new browser windows opening then wait for it to load.
	function windowWatcher(subject, topic) {
		if(topic == "domwindowopened") {
			runOnLoad(subject, watcher, winType);
		};
	};
	Services.ww.registerNotification(windowWatcher);

	// Make sure to stop watching for windows if we're unloading.
	unload(function() {
		Services.ww.unregisterNotification(windowWatcher);
	});
};
